@@ -16,6 +16,15 @@ use capstone::{
     Capstone, Insn,
 };
 
+#[cfg(all(target_arch = "x86_64", unix))]
+use capstone::{
+    arch::{self, x86::X86OperandType, ArchOperand::X86Operand, BuildsCapstone},
+    Capstone, Insn, RegId,
+};
+
+#[cfg(target_arch = "aarch64")]
+use num_traits::cast::FromPrimitive;
+
 #[cfg(target_arch = "x86_64")]
 use frida_gum::instruction_writer::X86Register;
 #[cfg(target_arch = "aarch64")]
@@ -23,18 +32,27 @@ use frida_gum::instruction_writer::{Aarch64Register, IndexMode};
 use frida_gum::{
     instruction_writer::InstructionWriter,
     stalker::{StalkerOutput, Transformer},
-    CpuContext, ModuleDetails, ModuleMap,
+    ModuleDetails, ModuleMap,
 };
+
+#[cfg(unix)]
+use frida_gum::CpuContext;
+
 use frida_gum::{Gum, Module, PageProtection};
-#[cfg(target_arch = "aarch64")]
-use num_traits::cast::FromPrimitive;
 
 use rangemap::RangeMap;
 
+#[cfg(unix)]
 use nix::sys::mman::{mmap, MapFlags, ProtFlags};
 
+#[cfg(unix)]
 use crate::{asan_rt::AsanRuntime, FridaOptions};
 
+#[cfg(windows)]
+use crate::FridaOptions;
+
+use crate::coverage_rt::CoverageRuntime;
+
 #[cfg(feature = "cmplog")]
 use crate::cmplog_rt::CmpLogRuntime;
 
@@ -46,17 +64,23 @@ enum CmplogOperandType {
     Mem(capstone::RegId, capstone::RegId, i32, u32),
 }
 
-#[cfg(target_vendor = "apple")]
+enum SpecialCmpLogCase {
+    Tbz,
+    Tbnz,
+}
+
+#[cfg(any(target_vendor = "apple"))]
 const ANONYMOUS_FLAG: MapFlags = MapFlags::MAP_ANON;
-#[cfg(not(target_vendor = "apple"))]
+#[cfg(not(any(target_vendor = "apple", target_os = "windows")))]
 const ANONYMOUS_FLAG: MapFlags = MapFlags::MAP_ANONYMOUS;
 
-/// An helper that feeds [`FridaInProcessExecutor`] with user-supplied instrumentation
+/// An helper that feeds `FridaInProcessExecutor` with user-supplied instrumentation
 pub trait FridaHelper<'a> {
     /// Access to the stalker `Transformer`
     fn transformer(&self) -> &Transformer<'a>;
 
     /// Register a new thread with this `FridaHelper`
+    #[cfg(unix)]
     fn register_thread(&mut self);
 
     /// Called prior to execution of an input
@@ -69,25 +93,21 @@ pub trait FridaHelper<'a> {
     fn stalker_enabled(&self) -> bool;
 
     /// pointer to the frida coverage map
-    fn map_ptr(&mut self) -> *mut u8;
+    fn map_ptr_mut(&mut self) -> *mut u8;
 
     fn ranges(&self) -> &RangeMap<usize, (u16, String)>;
 }
 
-/// (Default) map size for frida coverage reporting
-pub const MAP_SIZE: usize = 64 * 1024;
-
-/// An helper that feeds [`FridaInProcessExecutor`] with edge-coverage instrumentation
+/// An helper that feeds `FridaInProcessExecutor` with edge-coverage instrumentation
 pub struct FridaInstrumentationHelper<'a> {
-    map: [u8; MAP_SIZE],
-    previous_pc: [u64; 1],
-    current_log_impl: u64,
-    #[cfg(target_arch = "aarch64")]
+    coverage_rt: CoverageRuntime,
+    #[cfg(unix)]
     current_report_impl: u64,
     /// Transformer that has to be passed to FridaInProcessExecutor
     transformer: Option<Transformer<'a>>,
-    #[cfg(target_arch = "aarch64")]
+    #[cfg(unix)]
     capstone: Capstone,
+    #[cfg(unix)]
     asan_runtime: AsanRuntime,
     #[cfg(feature = "cmplog")]
     cmplog_runtime: CmpLogRuntime,
@@ -103,20 +123,19 @@ impl<'a> FridaHelper<'a> for FridaInstrumentationHelper<'a> {
     }
 
     /// Register the current thread with the [`FridaInstrumentationHelper`]
+    #[cfg(unix)]
     fn register_thread(&mut self) {
         self.asan_runtime.register_thread();
     }
 
-    #[cfg(not(target_arch = "aarch64"))]
+    #[cfg(not(unix))]
     fn pre_exec<I: Input + HasTargetBytes>(&mut self, _input: &I) {}
 
-    #[cfg(target_arch = "aarch64")]
+    #[cfg(unix)]
     fn pre_exec<I: Input + HasTargetBytes>(&mut self, input: &I) {
-        #[cfg(target_arch = "aarch64")]
         let target_bytes = input.target_bytes();
         let slice = target_bytes.as_slice();
         //println!("target_bytes: {:#x}: {:02x?}", slice.as_ptr() as usize, slice);
-        #[cfg(target_arch = "aarch64")]
         if self.options.asan_enabled() {
             self.asan_runtime
                 .unpoison(slice.as_ptr() as usize, slice.len());
@@ -132,7 +151,7 @@ impl<'a> FridaHelper<'a> for FridaInstrumentationHelper<'a> {
             DrCovWriter::new(&filename, &self.ranges, &mut self.drcov_basic_blocks).write();
         }
 
-        #[cfg(target_arch = "aarch64")]
+        #[cfg(unix)]
         if self.options.asan_enabled() {
             if self.options.asan_detect_leaks() {
                 self.asan_runtime.check_for_leaks();
@@ -150,8 +169,8 @@ impl<'a> FridaHelper<'a> for FridaInstrumentationHelper<'a> {
         self.options.stalker_enabled()
     }
 
-    fn map_ptr(&mut self) -> *mut u8 {
-        self.map.as_mut_ptr()
+    fn map_ptr_mut(&mut self) -> *mut u8 {
+        self.coverage_rt.map_ptr_mut()
     }
 
     fn ranges(&self) -> &RangeMap<usize, (u16, String)> {
@@ -172,70 +191,12 @@ pub fn get_module_size(module_name: &str) -> usize {
     code_size
 }
 
-/// A minimal `maybe_log` implementation. We insert this into the transformed instruction stream
-/// every time we need a copy that is within a direct branch of the start of the transformed basic
-/// block.
-#[cfg(target_arch = "x86_64")]
-const MAYBE_LOG_CODE: [u8; 47] = [
-    0x9c, /* pushfq */
-    0x50, /* push rax */
-    0x51, /* push rcx */
-    0x52, /* push rdx */
-    0x48, 0x8d, 0x05, 0x24, 0x00, 0x00, 0x00, /* lea rax, sym._afl_area_ptr_ptr */
-    0x48, 0x8b, 0x00, /* mov rax, qword [rax] */
-    0x48, 0x8d, 0x0d, 0x22, 0x00, 0x00, 0x00, /* lea rcx, sym.previous_pc     */
-    0x48, 0x8b, 0x11, /* mov rdx, qword [rcx] */
-    0x48, 0x8b, 0x12, /* mov rdx, qword [rdx] */
-    0x48, 0x31, 0xfa, /* xor rdx, rdi */
-    0xfe, 0x04, 0x10, /* inc byte [rax + rdx] */
-    0x48, 0xd1, 0xef, /* shr rdi, 1 */
-    0x48, 0x8b, 0x01, /* mov rax, qword [rcx] */
-    0x48, 0x89, 0x38, /* mov qword [rax], rdi */
-    0x5a, /* pop rdx */
-    0x59, /* pop rcx */
-    0x58, /* pop rax */
-    0x9d, /* popfq */
-    0xc3, /* ret */
-
-          /* Read-only data goes here: */
-          /* uint8_t* afl_area_ptr */
-          /* uint64_t* afl_prev_loc_ptr */
-];
-
-#[cfg(target_arch = "aarch64")]
-const MAYBE_LOG_CODE: [u8; 60] = [
-    // __afl_area_ptr[current_pc ^ previous_pc]++;
-    // previous_pc = current_pc >> 1;
-    0xE1, 0x0B, 0xBF, 0xA9, // stp x1, x2, [sp, -0x10]!
-    0xE3, 0x13, 0xBF, 0xA9, // stp x3, x4, [sp, -0x10]!
-    // x0 = current_pc
-    0xa1, 0x01, 0x00, 0x58, // ldr x1, #0x30, =__afl_area_ptr
-    0x82, 0x01, 0x00, 0x58, // ldr x2, #0x38, =&previous_pc
-    0x44, 0x00, 0x40, 0xf9, // ldr x4, [x2] (=previous_pc)
-    // __afl_area_ptr[current_pc ^ previous_pc]++;
-    0x84, 0x00, 0x00, 0xca, // eor x4, x4, x0
-    0x84, 0x3c, 0x40, 0x92, // and x4, x4, 0xffff (=MAP_SIZE - 1)
-    //0x20, 0x13, 0x20, 0xd4,
-    0x23, 0x68, 0x64, 0xf8, // ldr x3, [x1, x4]
-    0x63, 0x04, 0x00, 0x91, // add x3, x3, #1
-    0x23, 0x68, 0x24, 0xf8, // str x3, [x1, x4]
-    // previous_pc = current_pc >> 1;
-    0xe0, 0x07, 0x40, 0x8b, // add x0, xzr, x0, LSR #1
-    0x40, 0x00, 0x00, 0xf9, // str x0, [x2]
-    0xE3, 0x13, 0xc1, 0xA8, // ldp x3, x4, [sp], #0x10
-    0xE1, 0x0B, 0xc1, 0xA8, // ldp x1, x2, [sp], #0x10
-    0xC0, 0x03, 0x5F, 0xD6, // ret
-
-          // &afl_area_ptr
-          // &afl_prev_loc_ptr
-];
-
 #[cfg(target_arch = "aarch64")]
 fn pc(context: &CpuContext) -> usize {
     context.pc() as usize
 }
 
-#[cfg(target_arch = "x86_64")]
+#[cfg(all(target_arch = "x86_64", unix))]
 fn pc(context: &CpuContext) -> usize {
     context.rip() as usize
 }
@@ -252,6 +213,7 @@ impl<'a> FridaInstrumentationHelper<'a> {
         modules_to_instrument: &'a [&str],
     ) -> Self {
         // workaround frida's frida-gum-allocate-near bug:
+        #[cfg(unix)]
         unsafe {
             for _ in 0..512 {
                 mmap(
@@ -276,10 +238,8 @@ impl<'a> FridaInstrumentationHelper<'a> {
         }
 
         let mut helper = Self {
-            map: [0u8; MAP_SIZE],
-            previous_pc: [0u64; 1],
-            current_log_impl: 0,
-            #[cfg(target_arch = "aarch64")]
+            coverage_rt: CoverageRuntime::new(),
+            #[cfg(unix)]
             current_report_impl: 0,
             transformer: None,
             #[cfg(target_arch = "aarch64")]
@@ -289,6 +249,14 @@ impl<'a> FridaInstrumentationHelper<'a> {
                 .detail(true)
                 .build()
                 .expect("Failed to create Capstone object"),
+            #[cfg(all(target_arch = "x86_64", unix))]
+            capstone: Capstone::new()
+                .x86()
+                .mode(arch::x86::ArchMode::Mode64)
+                .detail(true)
+                .build()
+                .expect("Failed to create Capstone object"),
+            #[cfg(not(windows))]
             asan_runtime: AsanRuntime::new(options.clone()),
             #[cfg(feature = "cmplog")]
             cmplog_runtime: CmpLogRuntime::new(),
@@ -302,6 +270,7 @@ impl<'a> FridaInstrumentationHelper<'a> {
             for (i, module) in helper.module_map.values().iter().enumerate() {
                 let range = module.range();
                 let start = range.base_address().0 as usize;
+                // println!("start: {:x}", start);
                 helper
                     .ranges
                     .insert(start..(start + range.size()), (i as u16, module.path()));
@@ -310,7 +279,7 @@ impl<'a> FridaInstrumentationHelper<'a> {
                 for (module_name, offset) in suppressed_specifiers {
                     let module_details = ModuleDetails::with_name(module_name).unwrap();
                     let lib_start = module_details.range().base_address().0 as usize;
-                    println!("removing address: {:#x}", lib_start + offset);
+                    // println!("removing address: {:#x}", lib_start + offset);
                     helper
                         .ranges
                         .remove((lib_start + offset)..(lib_start + offset + 4));
@@ -322,20 +291,32 @@ impl<'a> FridaInstrumentationHelper<'a> {
                     .expect("failed to create directory for coverage files");
             }
 
+            if helper.options().coverage_enabled() {
+                helper.coverage_rt.init();
+            }
+
             let transformer = Transformer::from_callback(gum, |basic_block, output| {
                 let mut first = true;
                 for instruction in basic_block {
                     let instr = instruction.instr();
                     let address = instr.address();
-                    //println!("block @ {:x} transformed to {:x}", address, output.writer().pc());
-                    //println!("address: {:x} contains: {:?}", address, helper.ranges.contains_key(&(address as usize)));
+                    // println!("block @ {:x} transformed to {:x}", address, output.writer().pc());
+                    /*
+                    println!(
+                        "address: {:x} contains: {:?}",
+                        address,
+                        helper.ranges.contains_key(&(address as usize))
+                    );
+                    */
+                    // println!("Ranges: {:#?}", helper.ranges);
                     if helper.ranges.contains_key(&(address as usize)) {
                         if first {
                             first = false;
-                            //println!("block @ {:x} transformed to {:x}", address, output.writer().pc());
+                            // println!("block @ {:x} transformed to {:x}", address, output.writer().pc());
                             if helper.options().coverage_enabled() {
-                                helper.emit_coverage_mapping(address, &output);
+                                helper.coverage_rt.emit_coverage_mapping(address, &output);
                             }
+                            #[cfg(unix)]
                             if helper.options().drcov_enabled() {
                                 instruction.put_callout(|context| {
                                     let real_address =
@@ -350,8 +331,15 @@ impl<'a> FridaInstrumentationHelper<'a> {
                         }
 
                         if helper.options().asan_enabled() {
-                            #[cfg(not(target_arch = "aarch64"))]
-                            todo!("Implement ASAN for non-aarch64 targets");
+                            #[cfg(all(target_arch = "x86_64", unix))]
+                            if let Ok((segment, width, basereg, indexreg, scale, disp)) =
+                                helper.asan_is_interesting_instruction(address, instr)
+                            {
+                                helper.emit_shadow_check(
+                                    address, &output, segment, width, basereg, indexreg, scale,
+                                    disp,
+                                );
+                            }
                             #[cfg(target_arch = "aarch64")]
                             if let Ok((basereg, indexreg, displacement, width, shift, extender)) =
                                 helper.asan_is_interesting_instruction(address, instr)
@@ -373,14 +361,21 @@ impl<'a> FridaInstrumentationHelper<'a> {
                             todo!("Implement cmplog for non-aarch64 targets");
                             #[cfg(all(feature = "cmplog", target_arch = "aarch64"))]
                             // check if this instruction is a compare instruction and if so save the registers values
-                            if let Ok((op1, op2)) =
+                            if let Ok((op1, op2, special_case)) =
                                 helper.cmplog_is_interesting_instruction(address, instr)
                             {
                                 //emit code that saves the relevant data in runtime(passes it to x0, x1)
-                                helper.emit_comparison_handling(address, &output, op1, op2);
+                                helper.emit_comparison_handling(
+                                    address,
+                                    &output,
+                                    op1,
+                                    op2,
+                                    special_case,
+                                );
                             }
                         }
 
+                        #[cfg(unix)]
                         if helper.options().asan_enabled() || helper.options().drcov_enabled() {
                             helper.asan_runtime.add_stalked_address(
                                 output.writer().pc() as usize - 4,
@@ -392,6 +387,8 @@ impl<'a> FridaInstrumentationHelper<'a> {
                 }
             });
             helper.transformer = Some(transformer);
+
+            #[cfg(unix)]
             if helper.options().asan_enabled() || helper.options().drcov_enabled() {
                 helper.asan_runtime.init(gum, modules_to_instrument);
             }
@@ -414,6 +411,53 @@ impl<'a> FridaInstrumentationHelper<'a> {
         Aarch64Register::from_u32(regint as u32).unwrap()
     }
 
+    // frida registers: https://docs.rs/frida-gum/0.4.0/frida_gum/instruction_writer/enum.X86Register.html
+    // capstone registers: https://docs.rs/capstone-sys/0.14.0/capstone_sys/x86_reg/index.html
+    #[cfg(all(target_arch = "x86_64", unix))]
+    #[must_use]
+    #[inline]
+    #[allow(clippy::unused_self)]
+    pub fn writer_register(&self, reg: RegId) -> X86Register {
+        let regint: u16 = reg.0;
+        match regint {
+            19 => X86Register::Eax,
+            22 => X86Register::Ecx,
+            24 => X86Register::Edx,
+            21 => X86Register::Ebx,
+            30 => X86Register::Esp,
+            20 => X86Register::Ebp,
+            29 => X86Register::Esi,
+            23 => X86Register::Edi,
+            226 => X86Register::R8d,
+            227 => X86Register::R9d,
+            228 => X86Register::R10d,
+            229 => X86Register::R11d,
+            230 => X86Register::R12d,
+            231 => X86Register::R13d,
+            232 => X86Register::R14d,
+            233 => X86Register::R15d,
+            26 => X86Register::Eip,
+            35 => X86Register::Rax,
+            38 => X86Register::Rcx,
+            40 => X86Register::Rdx,
+            37 => X86Register::Rbx,
+            44 => X86Register::Rsp,
+            36 => X86Register::Rbp,
+            43 => X86Register::Rsi,
+            39 => X86Register::Rdi,
+            106 => X86Register::R8,
+            107 => X86Register::R9,
+            108 => X86Register::R10,
+            109 => X86Register::R11,
+            110 => X86Register::R12,
+            111 => X86Register::R13,
+            112 => X86Register::R14,
+            113 => X86Register::R15,
+            41 => X86Register::Rip,
+            _ => X86Register::None, // Ignore Xax..Xip
+        }
+    }
+
     #[cfg(all(feature = "cmplog", target_arch = "aarch64"))]
     #[inline]
     /// Emit the instrumentation code which is responsible for opernads value extraction and cmplog map population
@@ -423,6 +467,7 @@ impl<'a> FridaInstrumentationHelper<'a> {
         output: &StalkerOutput,
         op1: CmplogOperandType,
         op2: CmplogOperandType,
+        special_case: Option<SpecialCmpLogCase>,
     ) {
         let writer = output.writer();
 
@@ -504,6 +549,17 @@ impl<'a> FridaInstrumentationHelper<'a> {
         match op2 {
             CmplogOperandType::Imm(value) | CmplogOperandType::Cimm(value) => {
                 writer.put_ldr_reg_u64(Aarch64Register::X1, value);
+                match special_case {
+                    Some(inst) => match inst {
+                        SpecialCmpLogCase::Tbz => {
+                            writer.put_bytes(&self.cmplog_runtime.ops_handle_tbz_masking());
+                        }
+                        SpecialCmpLogCase::Tbnz => {
+                            writer.put_bytes(&self.cmplog_runtime.ops_handle_tbnz_masking());
+                        }
+                    },
+                    None => (),
+                }
             }
             CmplogOperandType::Regid(reg) => {
                 let reg = self.writer_register(reg);
@@ -720,6 +776,161 @@ impl<'a> FridaInstrumentationHelper<'a> {
         ));
     }
 
+    #[inline]
+    #[allow(clippy::too_many_lines)]
+    #[allow(clippy::too_many_arguments)]
+    #[cfg(all(target_arch = "x86_64", unix))]
+    pub fn emit_shadow_check(
+        &mut self,
+        address: u64,
+        output: &StalkerOutput,
+        _segment: RegId,
+        width: u8,
+        basereg: RegId,
+        indexreg: RegId,
+        scale: i32,
+        disp: i64,
+    ) {
+        let redzone_size = i64::from(frida_gum_sys::GUM_RED_ZONE_SIZE);
+        let writer = output.writer();
+        let true_rip = address;
+
+        let basereg = if basereg.0 == 0 {
+            None
+        } else {
+            let reg = self.writer_register(basereg);
+            Some(reg)
+        };
+
+        let indexreg = if indexreg.0 == 0 {
+            None
+        } else {
+            let reg = self.writer_register(indexreg);
+            Some(reg)
+        };
+
+        let scale = match scale {
+            2 => 1,
+            4 => 2,
+            8 => 3,
+            _ => 0,
+        };
+        if self.current_report_impl == 0
+            || !writer.can_branch_directly_to(self.current_report_impl)
+            || !writer.can_branch_directly_between(writer.pc() + 128, self.current_report_impl)
+        {
+            let after_report_impl = writer.code_offset() + 2;
+
+            #[cfg(target_arch = "x86_64")]
+            writer.put_jmp_near_label(after_report_impl);
+            #[cfg(target_arch = "aarch64")]
+            writer.put_b_label(after_report_impl);
+
+            self.current_report_impl = writer.pc();
+            #[cfg(unix)]
+            writer.put_bytes(self.asan_runtime.blob_report());
+
+            writer.put_label(after_report_impl);
+        }
+
+        /* Save registers that we'll use later in shadow_check_blob
+                                        | addr  | rip   |
+                                        | Rcx   | Rax   |
+                                        | Rsi   | Rdx   |
+            Old Rsp - (redsone_size) -> | flags | Rdi   |
+                                        |       |       |
+            Old Rsp                  -> |       |       |
+        */
+        writer.put_lea_reg_reg_offset(X86Register::Rsp, X86Register::Rsp, -(redzone_size));
+        writer.put_pushfx();
+        writer.put_push_reg(X86Register::Rdi);
+        writer.put_push_reg(X86Register::Rsi);
+        writer.put_push_reg(X86Register::Rdx);
+        writer.put_push_reg(X86Register::Rcx);
+        writer.put_push_reg(X86Register::Rax);
+
+        /*
+        Things are a bit different when Rip is either base register or index register.
+        Suppose we have an instruction like
+        `bnd jmp qword ptr [rip + 0x2e4b5]`
+        We can't just emit code like
+        `mov rdi, rip` to get RIP loaded into RDI,
+        because this RIP is NOT the orginal RIP (, which is usually within .text) anymore, rather it is pointing to the memory allocated by the frida stalker.
+        Please confer https://frida.re/docs/stalker/ for details.
+        */
+        // Init Rdi
+        match basereg {
+            Some(reg) => match reg {
+                X86Register::Rip => {
+                    writer.put_mov_reg_address(X86Register::Rdi, true_rip);
+                }
+                _ => {
+                    writer.put_mov_reg_reg(X86Register::Rdi, basereg.unwrap());
+                }
+            },
+            None => {
+                writer.put_xor_reg_reg(X86Register::Rdi, X86Register::Rdi);
+            }
+        }
+
+        match indexreg {
+            Some(reg) => match reg {
+                X86Register::Rip => {
+                    writer.put_mov_reg_address(X86Register::Rsi, true_rip);
+                }
+                _ => {
+                    writer.put_mov_reg_reg(X86Register::Rsi, indexreg.unwrap());
+                }
+            },
+            None => {
+                writer.put_xor_reg_reg(X86Register::Rsi, X86Register::Rsi);
+            }
+        }
+
+        // Scale
+        if scale > 0 {
+            writer.put_shl_reg_u8(X86Register::Rsi, scale);
+        }
+
+        // Finally set Rdi to base + index * scale + disp
+        writer.put_add_reg_reg(X86Register::Rdi, X86Register::Rsi);
+        writer.put_lea_reg_reg_offset(X86Register::Rdi, X86Register::Rdi, disp);
+
+        writer.put_mov_reg_address(X86Register::Rsi, true_rip); // load true_rip into rsi in case we need them in handle_trap
+        writer.put_push_reg(X86Register::Rsi); // save true_rip
+        writer.put_push_reg(X86Register::Rdi); // save accessed_address
+
+        #[cfg(unix)]
+        let checked: bool = match width {
+            1 => writer.put_bytes(self.asan_runtime.blob_check_mem_byte()),
+            2 => writer.put_bytes(self.asan_runtime.blob_check_mem_halfword()),
+            4 => writer.put_bytes(self.asan_runtime.blob_check_mem_dword()),
+            8 => writer.put_bytes(self.asan_runtime.blob_check_mem_qword()),
+            16 => writer.put_bytes(self.asan_runtime.blob_check_mem_16bytes()),
+            _ => false,
+        };
+
+        if checked {
+            writer.put_jmp_address(self.current_report_impl);
+            for _ in 0..10 {
+                // shadow_check_blob's done will land somewhere in these nops
+                // on amd64 jump can takes 10 bytes at most, so that's why I put 10 bytes.
+                writer.put_nop();
+            }
+        }
+
+        writer.put_pop_reg(X86Register::Rdi);
+        writer.put_pop_reg(X86Register::Rsi);
+
+        writer.put_pop_reg(X86Register::Rax);
+        writer.put_pop_reg(X86Register::Rcx);
+        writer.put_pop_reg(X86Register::Rdx);
+        writer.put_pop_reg(X86Register::Rsi);
+        writer.put_pop_reg(X86Register::Rdi);
+        writer.put_popfx();
+        writer.put_lea_reg_reg_offset(X86Register::Rsp, X86Register::Rsp, redzone_size);
+    }
+
     #[cfg(target_arch = "aarch64")]
     #[inline]
     fn emit_shadow_check(
@@ -755,6 +966,8 @@ impl<'a> FridaInstrumentationHelper<'a> {
             writer.put_b_label(after_report_impl);
 
             self.current_report_impl = writer.pc();
+
+            #[cfg(unix)]
             writer.put_bytes(self.asan_runtime.blob_report());
 
             writer.put_label(after_report_impl);
@@ -891,6 +1104,7 @@ impl<'a> FridaInstrumentationHelper<'a> {
             }
         }
         // Insert the check_shadow_mem code blob
+        #[cfg(unix)]
         match width {
             1 => writer.put_bytes(&self.asan_runtime.blob_check_mem_byte()),
             2 => writer.put_bytes(&self.asan_runtime.blob_check_mem_halfword()),
@@ -1047,6 +1261,68 @@ impl<'a> FridaInstrumentationHelper<'a> {
         Err(())
     }
 
+    #[cfg(all(target_arch = "x86_64", unix))]
+    #[inline]
+    fn asan_is_interesting_instruction(
+        &self,
+        _address: u64,
+        instr: &Insn,
+    ) -> Result<(RegId, u8, RegId, RegId, i32, i64), ()> {
+        let operands = self
+            .capstone
+            .insn_detail(instr)
+            .unwrap()
+            .arch_detail()
+            .operands();
+
+        // Ignore lea instruction
+        // put nop into the white-list so that instructions like
+        // like `nop dword [rax + rax]` does not get caught.
+        match instr.mnemonic().unwrap() {
+            "lea" | "nop" => return Err(()),
+
+            _ => (),
+        }
+
+        // This is a TODO! In this case, both the src and the dst are mem operand
+        // so we would need to return two operadns?
+        if instr.mnemonic().unwrap().starts_with("rep") {
+            return Err(());
+        }
+
+        for operand in operands {
+            if let X86Operand(x86operand) = operand {
+                if let X86OperandType::Mem(opmem) = x86operand.op_type {
+                    /*
+                    println!(
+                        "insn: {:#?} {:#?} width: {}, segment: {:#?}, base: {:#?}, index: {:#?}, scale: {}, disp: {}",
+                        insn_id,
+                        instr,
+                        x86operand.size,
+                        opmem.segment(),
+                        opmem.base(),
+                        opmem.index(),
+                        opmem.scale(),
+                        opmem.disp(),
+                    );
+                    */
+                    if opmem.segment() == RegId(0) {
+                        return Ok((
+                            opmem.segment(),
+                            x86operand.size,
+                            opmem.base(),
+                            opmem.index(),
+                            opmem.scale(),
+                            opmem.disp(),
+                        ));
+                    }
+                }
+            }
+        }
+
+        Err(())
+    }
+
     #[cfg(all(feature = "cmplog", target_arch = "aarch64"))]
     #[inline]
     /// Check if the current instruction is cmplog relevant one(any opcode which sets the flags)
@@ -1054,22 +1330,48 @@ impl<'a> FridaInstrumentationHelper<'a> {
         &self,
         _address: u64,
         instr: &Insn,
-    ) -> Result<(CmplogOperandType, CmplogOperandType), ()> {
+    ) -> Result<
+        (
+            CmplogOperandType,
+            CmplogOperandType,
+            Option<SpecialCmpLogCase>,
+        ),
+        (),
+    > {
         // We only care for compare instrunctions - aka instructions which set the flags
         match instr.mnemonic().unwrap() {
-            "cmp" | "ands" | "subs" | "adds" | "negs" | "ngcs" | "sbcs" | "bics" | "cls" => (),
+            "cmp" | "ands" | "subs" | "adds" | "negs" | "ngcs" | "sbcs" | "bics" | "cbz"
+            | "cbnz" | "tbz" | "tbnz" | "adcs" => (),
             _ => return Err(()),
         }
-        let operands = self
+        let mut operands = self
             .capstone
             .insn_detail(instr)
             .unwrap()
             .arch_detail()
             .operands();
-        if operands.len() != 2 {
+
+        // cbz - 1 operand, tbz - 3 operands
+        let special_case = [
+            "cbz", "cbnz", "tbz", "tbnz", "subs", "adds", "ands", "sbcs", "bics", "adcs",
+        ]
+        .contains(&instr.mnemonic().unwrap());
+        if operands.len() != 2 && !special_case {
             return Err(());
         }
 
+        // handle special opcodes case which have 3 operands, but the 1st(dest) is not important to us
+        if ["subs", "adds", "ands", "sbcs", "bics", "adcs"].contains(&instr.mnemonic().unwrap()) {
+            //remove the dest operand from the list
+            operands.remove(0);
+        }
+
+        // cbz marked as special since there is only 1 operand
+        let special_case = match instr.mnemonic().unwrap() {
+            "cbz" | "cbnz" => true,
+            _ => false,
+        };
+
         let operand1 = if let Arm64Operand(arm64operand) = operands.first().unwrap() {
             match arm64operand.op_type {
                 Arm64OperandType::Reg(regid) => Some(CmplogOperandType::Regid(regid)),
@@ -1087,89 +1389,39 @@ impl<'a> FridaInstrumentationHelper<'a> {
             None
         };
 
-        let operand2 = if let Arm64Operand(arm64operand2) = operands.last().unwrap() {
-            match arm64operand2.op_type {
-                Arm64OperandType::Reg(regid) => Some(CmplogOperandType::Regid(regid)),
-                Arm64OperandType::Imm(val) => Some(CmplogOperandType::Imm(val as u64)),
-                Arm64OperandType::Mem(opmem) => Some(CmplogOperandType::Mem(
-                    opmem.base(),
-                    opmem.index(),
-                    opmem.disp(),
-                    self.instruction_width(instr, &operands),
-                )),
-                Arm64OperandType::Cimm(val) => Some(CmplogOperandType::Cimm(val as u64)),
-                _ => return Err(()),
+        let operand2 = match special_case {
+            true => Some(CmplogOperandType::Imm(0)),
+            false => {
+                if let Arm64Operand(arm64operand2) = &operands[1] {
+                    match arm64operand2.op_type {
+                        Arm64OperandType::Reg(regid) => Some(CmplogOperandType::Regid(regid)),
+                        Arm64OperandType::Imm(val) => Some(CmplogOperandType::Imm(val as u64)),
+                        Arm64OperandType::Mem(opmem) => Some(CmplogOperandType::Mem(
+                            opmem.base(),
+                            opmem.index(),
+                            opmem.disp(),
+                            self.instruction_width(instr, &operands),
+                        )),
+                        Arm64OperandType::Cimm(val) => Some(CmplogOperandType::Cimm(val as u64)),
+                        _ => return Err(()),
+                    }
+                } else {
+                    None
+                }
             }
-        } else {
-            None
+        };
+
+        // tbz will need to have special handling at emit time(masking operand1 value with operand2)
+        let special_case = match instr.mnemonic().unwrap() {
+            "tbz" => Some(SpecialCmpLogCase::Tbz),
+            "tbnz" => Some(SpecialCmpLogCase::Tbnz),
+            _ => None,
         };
 
         if operand1.is_some() && operand2.is_some() {
-            Ok((operand1.unwrap(), operand2.unwrap()))
+            Ok((operand1.unwrap(), operand2.unwrap(), special_case))
         } else {
             Err(())
         }
     }
-
-    #[inline]
-    fn emit_coverage_mapping(&mut self, address: u64, output: &StalkerOutput) {
-        let writer = output.writer();
-        #[allow(clippy::cast_possible_wrap)] // gum redzone size is u32, we need an offset as i32.
-        let redzone_size = frida_gum_sys::GUM_RED_ZONE_SIZE as i32;
-        if self.current_log_impl == 0
-            || !writer.can_branch_directly_to(self.current_log_impl)
-            || !writer.can_branch_directly_between(writer.pc() + 128, self.current_log_impl)
-        {
-            let after_log_impl = writer.code_offset() + 1;
-
-            #[cfg(target_arch = "x86_64")]
-            writer.put_jmp_near_label(after_log_impl);
-            #[cfg(target_arch = "aarch64")]
-            writer.put_b_label(after_log_impl);
-
-            self.current_log_impl = writer.pc();
-            writer.put_bytes(&MAYBE_LOG_CODE);
-            let prev_loc_pointer = self.previous_pc.as_ptr() as usize;
-            let map_pointer = self.map.as_ptr() as usize;
-
-            writer.put_bytes(&map_pointer.to_ne_bytes());
-            writer.put_bytes(&prev_loc_pointer.to_ne_bytes());
-
-            writer.put_label(after_log_impl);
-        }
-        #[cfg(target_arch = "x86_64")]
-        {
-            writer.put_lea_reg_reg_offset(X86Register::Rsp, X86Register::Rsp, -(redzone_size));
-            writer.put_push_reg(X86Register::Rdi);
-            writer.put_mov_reg_address(
-                X86Register::Rdi,
-                ((address >> 4) ^ (address << 8)) & (MAP_SIZE - 1) as u64,
-            );
-            writer.put_call_address(self.current_log_impl);
-            writer.put_pop_reg(X86Register::Rdi);
-            writer.put_lea_reg_reg_offset(X86Register::Rsp, X86Register::Rsp, redzone_size);
-        }
-        #[cfg(target_arch = "aarch64")]
-        {
-            writer.put_stp_reg_reg_reg_offset(
-                Aarch64Register::Lr,
-                Aarch64Register::X0,
-                Aarch64Register::Sp,
-                -(16 + redzone_size) as i64,
-                IndexMode::PreAdjust,
-            );
-            writer.put_ldr_reg_u64(
-                Aarch64Register::X0,
-                ((address >> 4) ^ (address << 8)) & (MAP_SIZE - 1) as u64,
-            );
-            writer.put_bl_imm(self.current_log_impl);
-            writer.put_ldp_reg_reg_reg_offset(
-                Aarch64Register::Lr,
-                Aarch64Register::X0,
-                Aarch64Register::Sp,
-                16 + redzone_size as i64,
-                IndexMode::PostAdjust,
-            );
-        }
-    }
 }
