@@ -4,12 +4,17 @@ It can report coverage and, on supported architecutres, even reports memory acce
 */
 
 /// The frida-asan allocator
+#[cfg(unix)]
 pub mod alloc;
 /// Handling of ASAN errors
+#[cfg(unix)]
 pub mod asan_errors;
 /// The frida address sanitizer runtime
+#[cfg(unix)]
 pub mod asan_rt;
 
+pub mod coverage_rt;
+
 #[cfg(feature = "cmplog")]
 /// The frida cmplog runtime
 pub mod cmplog_rt;
@@ -17,6 +22,9 @@ pub mod cmplog_rt;
 /// The `LibAFL` firda helper
 pub mod helper;
 
+/// The frida executor
+pub mod executor;
+
 // for parsing asan and cmplog cores
 use libafl::bolts::os::parse_core_bind_arg;
 // for getting current core_id
@@ -39,7 +47,7 @@ pub struct FridaOptions {
 }
 
 impl FridaOptions {
-    /// Parse the frida options from the [`LIBAFL_FRIDA_OPTIONS`] environment variable.
+    /// Parse the frida options from the "`LIBAFL_FRIDA_OPTIONS`" environment variable.
     ///
     /// Options are `:` separated, and each options is a `name=value` string.
     ///
@@ -60,10 +68,6 @@ impl FridaOptions {
                 match name {
                     "asan" => {
                         options.enable_asan = value.parse().unwrap();
-                        #[cfg(not(target_arch = "aarch64"))]
-                        if options.enable_asan {
-                            panic!("ASAN is not currently supported on targets other than aarch64");
-                        }
                     }
                     "asan-detect-leaks" => {
                         options.enable_asan_leak_detection = value.parse().unwrap();
@@ -110,23 +114,21 @@ impl FridaOptions {
                     "drcov" => {
                         options.enable_drcov = value.parse().unwrap();
                         #[cfg(not(target_arch = "aarch64"))]
-                        if options.enable_drcov {
-                            panic!(
-                                "DrCov is not currently supported on targets other than aarch64"
-                            );
-                        }
+                        assert!(
+                            !options.enable_drcov,
+                            "DrCov is not currently supported on targets other than aarch64"
+                        );
                     }
                     "cmplog" => {
                         options.enable_cmplog = value.parse().unwrap();
                         #[cfg(not(target_arch = "aarch64"))]
-                        if options.enable_cmplog {
-                            panic!(
-                                "cmplog is not currently supported on targets other than aarch64"
-                            );
-                        }
+                        assert!(
+                            !options.enable_cmplog,
+                            "cmplog is not currently supported on targets other than aarch64"
+                        );
 
-                        if !cfg!(feature = "cmplog") && options.enable_cmplog {
-                            panic!("cmplog feature is disabled!");
+                        if options.enable_cmplog {
+                            assert!(cfg!(feature = "cmplog"), "cmplog feature is disabled!");
                         }
                     }
                     "cmplog-cores" => {
